<?php
//
// UTILITY FUNCTIONS
//

add_theme_support( 'custom-logo' );
add_theme_support( 'post-thumbnails' );
add_theme_support( 'title-tag' );

//  Custom URL on logo
add_filter( 'get_custom_logo', 'add_custom_logo_url' );
function add_custom_logo_url() {
    $custom_logo_id = get_theme_mod( 'custom_logo' );
    $logo_url = esc_url( home_url( '/' ) );

    if (get_theme_mod('logo_url', '0') > 0) {
      $logo_url = get_permalink(get_theme_mod('logo_url'));
    }

    $html = sprintf( '<a href="%1$s" class="custom-logo-link" rel="home" itemprop="url">%2$s</a>',
            $logo_url,
            wp_get_attachment_image( $custom_logo_id, 'full', false, array(
                'class'    => 'custom-logo',
                'alt'      => get_bloginfo( 'name', 'display' ),
            ) )
        );
    return $html;   
} 

//  Fix WPAdverts single category view
add_action("init", "gecko_theme_wpadverts_init", 20);
function gecko_theme_wpadverts_init() {
    remove_filter('template_include', 'adverts_template_include');
}

//  Detect blog pages
function is_blog() {
  return ( is_archive() || is_author() || is_category() || is_single() || is_tag()) && 'post' == get_post_type();
}

//  Get proper id for any page/post
function get_proper_ID() {
  $page_id = get_the_ID();

  if ( !is_front_page() && is_home() ) {
    $page_id = ( 'page' == get_option( 'show_on_front' ) ? get_option( 'page_for_posts' ) : get_the_ID );
  }

  if ( class_exists( 'woocommerce' ) ) {
    if (is_shop()) {
      $page_id = get_option( 'woocommerce_shop_page_id' );
    }
  }

  return $page_id;
}

//  Remove shortcodes from search results
function remove_shortcode_from_search( $content ) {
  if ( is_search() ) {
    $content = strip_shortcodes( $content );
  }
  return $content;
}
add_filter( 'the_content', 'remove_shortcode_from_search' );

//  Support WooCommerce
function gecko_add_woocommerce_support() {
  add_theme_support( 'woocommerce' );
}
add_action( 'after_setup_theme', 'gecko_add_woocommerce_support' );

add_theme_support( 'wc-product-gallery-zoom' );
add_theme_support( 'wc-product-gallery-lightbox' );
add_theme_support( 'wc-product-gallery-slider' );

//  WP admin bar visibility
$settings = GeckoConfigSettings::get_instance();
if (1==$settings->get_option( 'opt_show_adminbar', 0 ) ) : show_admin_bar( true ); endif;
if (2==$settings->get_option( 'opt_show_adminbar', 0 ) && ! current_user_can( 'manage_options' ) ) : show_admin_bar( false ); endif;
if (3==$settings->get_option( 'opt_show_adminbar', 0 ) ) : show_admin_bar( false ); endif;

function gecko_blocks()
{
    wp_register_script(
        'gecko-blocks',
        get_template_directory_uri() . '/assets/js/blocks.js',
        array('wp-blocks', 'wp-element', 'wp-editor'),
        wp_get_theme()->version
    );

    wp_register_style(
        'gecko-blocks',
        get_template_directory_uri() . '/assets/css/blocks.css',
        array(),
        wp_get_theme()->version
    );

    // register_block_type() was introduced in WP 5
    if (function_exists('register_block_type')) {
        register_block_type('gecko-blocks/container', array(
            'editor_script' => 'gecko-blocks',
            'editor_style' => 'gecko-blocks',
            //'style' => 'gecko-blocks'
        ));
    }
}
add_action( 'init', 'gecko_blocks' );

//  Get peepso color template
function get_peepso_color_template() {
  $color = "";

  if (class_exists( 'PeepSo' )) {
    $color = PeepSo::get_option('site_css_template','');
  }

  return $color;
}

//  Open Graph
add_action('wp_head', 'fc_opengraph', 100);
function fc_opengraph() {

  if( is_single() || is_page() ) {

    $post_id = get_queried_object_id();

    $url = get_permalink($post_id);
    $title = get_the_title($post_id);
    $site_name = get_bloginfo('name');

    $description = get_bloginfo('description');

    $locale = get_locale();

    echo '<!-- OpenGraph -->';
    echo '<meta property="og:locale" content="' . esc_attr($locale) . '" />';
    echo '<meta property="og:type" content="article" />';
    echo '<meta property="og:title" content="' . esc_attr($site_name) . ' | ' . esc_attr($title) . '" />';
    echo '<meta property="og:description" content="' . esc_attr($description) . '" />';
    echo '<meta property="og:url" content="' . esc_url($url) . '" />';
    echo '<meta property="og:site_name" content="' . esc_attr($site_name) . '" />';

    if( !has_post_thumbnail( $post_id ) ) {
      if ( has_custom_logo() ) {
        $custom_logo_id = get_theme_mod( 'custom_logo' );
        $logo = wp_get_attachment_image_src( $custom_logo_id , 'full' );
        $default_image = esc_url( $logo[0] );
      } else {
        $default_image = get_stylesheet_directory_uri() . "/assets/images/logo.svg";
      }

      echo '<meta property="og:image" content="' . $default_image . '"/>';
    } else {
      $default_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post_id ), 'medium' );
      echo '<meta property="og:image" content="' . esc_attr( $default_image[0] ) . '"/>';
    }

    echo '
    ';
  }

}

if ( ! function_exists( 'gc_post_date' ) ) {
  function gc_post_date() {
    if ( in_array( get_post_type(), array( 'post', 'attachment' ) ) ) {
      $settings = GeckoConfigSettings::get_instance();

      if (get_post_meta(get_proper_ID(), 'gecko-post-update-date', true) == 1) {
        $value = 0;
      } elseif (get_post_meta(get_proper_ID(), 'gecko-post-update-date', true) == 2) {
        $value = 1;
      } else {
        $value = $settings->get_option( 'opt_hide_blog_update', 0 );
      }

      $time_string = '<strong datetime="%1$s">%2$s</strong>';

      if ( get_the_time( 'U' ) !== get_the_modified_time( 'U' ) && $value == 0 ) {
        $time_string = '<strong datetime="%1$s">%2$s</strong> <span class="updated" datetime="%3$s">(updated %4$s)</span>';
      }

      $time_string = sprintf( $time_string,
        esc_attr( get_the_date( 'c' ) ),
        get_the_date(),
        esc_attr( get_the_modified_date( 'c' ) ),
        get_the_modified_date()
      );

      echo $time_string;
    }
  }
}
